

from typing import List, Tuple, Optional, AsyncGenerator
from reactivestreams.publisher import Publisher
from reactivestreams.subscriber import Subscriber
from reactivestreams.subscription import Subscription
from rsocket.streams.stream_from_async_generator import StreamFromAsyncGenerator
from rsocket.fragment import Fragment
from rsocket.payload import Payload
import os

from appbridge.prototol import Msg, Action
from appbridge import common

   
def publisher(files_dir, file_related_path) -> Publisher:
    full_path = os.path.join(files_dir, file_related_path)
    norm_path = os.path.normpath(full_path)
    basename = os.path.basename(norm_path)
    dir = os.path.dirname(norm_path)
    async def generator() -> AsyncGenerator[Tuple[Fragment, bool], None]:

        # send file
        async for e in  common.file_generator(dir, basename):
            yield Fragment(e), False

        # indicate channel complete
        msg = Msg(Action.Complete, [])
        yield Fragment(msg.toBytes), True
        
        print("publisher finished!")

    return StreamFromAsyncGenerator(generator)


# params: file_related_path
def handle_action(files_dir: str, params: List[str]) -> Tuple[Optional[Publisher], Optional[Subscriber]]:
    file_related_path = params[0]
    print('will send file/dir \"%s\" to clinet' % file_related_path)
    return publisher(files_dir, file_related_path), None
