
from asyncio import Event
from typing import List, Tuple, Optional, AsyncGenerator
from reactivestreams.publisher import Publisher
from reactivestreams.subscriber import Subscriber
from reactivestreams.subscription import Subscription
from rsocket.streams.stream_from_async_generator import StreamFromAsyncGenerator
from rsocket.fragment import Fragment
from rsocket.payload import Payload

from appbridge.progress_bar import TranferBar
from appbridge.prototol import Msg, Action
from appbridge import common

class ChannelSubscriber(Subscriber):

    file_received_size: int = 0
    file_total_size: int = 0
    file = None

    def __init__(self, files_dir: str, got_file_event: Event ) -> None:
        super().__init__()
        self.files_dir = files_dir
        self.got_file_event = got_file_event

    def __del__(self):
        print('Channel subscriber clear')
        if self.file:
            self.file.close()

    def on_subscribe(self, subscription: Subscription):
        self.subscription = subscription
        subscription.request(1)

    def on_error(self, exception: Exception):
        print('Error from client on channel' + str(exception))

    def on_complete(self):
        print('Completed from client on channel')

    def on_next(self, value: Payload, is_complete=False):

        # print("From client: len=%d, complete=%s" % (len(value.data), "Yes" if is_complete else "No"))

        # complete signal
        datalen = len(value.data)
        if datalen == 0 and is_complete:
            print("Skip complete empty msg")
            return

        # handle file receiving
        if self.file_received_size < self.file_total_size:
            self.file.write(value.data)
            self.file_received_size += datalen
            self.subscription.request(1)
            self.bar.next(datalen)
            return

        # handle msg
        msg = Msg.fromBytes(value.data)
        print('\nMsg: %s' % msg.toString())

        if msg.action == Action.ResponseOneFileInfo:
            (self.file, self.file_total_size) = common.prepare_receive_file(msg, self.files_dir)
            self.file_received_size = 0
            self.bar = TranferBar('Receiving', max=self.file_total_size)
            print("Prepared to receive a file!")

        elif msg.action == Action.ResponseOneFileComplete:
            self.file.close()
            self.file = None
            self.file_received_size = 0
            self.file_total_size = 0
            self.bar.finish()
            print("Received a file!")

        if msg.action == Action.ResponseAllFilesComplete:
            self.got_file_event.set()
        
        self.subscription.request(1)
        

    def publisher(self, filepath) -> Publisher:
        async def generator() -> AsyncGenerator[Tuple[Fragment, bool], None]:

            # request file
            req = Msg(Action.RequestFile, [filepath])
            yield Fragment(req.toBytes()), False

            # wait for file received
            await self.got_file_event.wait()
            msg = Msg(Action.Complete, [])
            yield Fragment(msg.toBytes()), True

        return StreamFromAsyncGenerator(generator)

# params: filepath
def handle_action(files_dir: str, params: List[str]) -> Tuple[Optional[Publisher], Optional[Subscriber]]:
    # print("handle send text")
    filepath = params[0]
    print('will receive file %s from clinet' % filepath)

    request_file_event = Event()
    subscriber = ChannelSubscriber(files_dir, request_file_event)

    return subscriber.publisher(filepath), subscriber
