


import logging
from asyncio import Future
import importlib.util
import os

from quart import Quart
from typing import Tuple, Optional
from rsocket.helpers import create_future
from rsocket.payload import Payload
from rsocket.request_handler import BaseRequestHandler
from rsocket.transports.quart_websocket import websocket_handler
from reactivestreams.publisher import Publisher
from reactivestreams.subscriber import Subscriber
from reactivestreams.subscription import Subscription
from rsocket.streams.stream_from_async_generator import StreamFromAsyncGenerator
from rsocket.fragment import Fragment

from appbridge import bonjour, request_file, send_file
from appbridge.prototol import Action, Msg

server_dir: str = ''
files_dir: str = ''

def custom_request_handler(data: Msg) -> Tuple[Optional[Publisher], Optional[Subscriber]]:
    
    # print("custom request handler")
    action = data.action
    # print("custom request handler1 %s" % action)

    module_file_path = server_dir + '/' + action + '.py'
    module_name = action

    if server_dir == '' or not os.path.isfile(module_file_path):
        if files_dir != '':
            if action == 'get':
                return request_file.handle_action(files_dir, data.params)
            elif action == 'send':
                return send_file.handle_action(files_dir, data.params)
        raise Exception("Server don't support action \"%s\"" % action)

    print("load %s from %s" % (module_name, module_file_path))
    module_spec = importlib.util.spec_from_file_location(module_name , module_file_path)
    module = importlib.util.module_from_spec(module_spec)
    module_spec.loader.exec_module(module)
    return module.handle_action(server_dir, data.params)


class Handler(BaseRequestHandler):

    async def request_response(self, payload: Payload) -> Future:
        # print(payload)

        msg = Msg.fromBytes(payload.data)

        if msg.action == Action.GetInfo:
            ret = Msg('', [])
            if server_dir != '' and os.path.isdir(server_dir):
                for item in os.listdir(server_dir):
                    if item.endswith('.py'):
                        ret.params.append(os.path.splitext(item)[0])
            if files_dir != '':
                ret.params.append('send')
                ret.params.append('get')
            return create_future(Payload(ret.toBytes()))
        
        # otherwise echo
        return create_future(payload)

    async def request_channel(self, payload: Payload) -> Tuple[Optional[Publisher], Optional[Subscriber]]:
        print(payload)
      
        # if payload.data is None:
        #     print("payload data is none")
        # print("ohh")
        # print(payload.metadata)
        # print(payload.data)
        data = Msg.fromBytes(payload.data)
        print("got request: %s" % data.toString())
        return custom_request_handler(data)
       
        
def start_service(name, dir, fdir):

    global server_dir
    server_dir = dir

    global files_dir
    files_dir = fdir

    port = 3000
    info = bonjour.register_service(name, port)
    name = info.name.split(".")[0]

    print("start service %s. service_dir=%s, files_dir=%s" % (name ,dir ,fdir)) 
    app = Quart(name)

    @app.websocket("/")
    async def ws():
        await websocket_handler(handler_factory=Handler)

    logging.basicConfig(level=logging.ERROR)
    
    while port < 9999:
        try:
            app.run('0.0.0.0', port=port)
            print("app end!")
            break
        except KeyboardInterrupt:
            break
        except:
            print("failed to run on port %d" % (port))
            port+=1
            # bonjour.unregister_service(info)
            info.port = port
            bonjour.update_service(info)

    bonjour.unregister_service(info)
    
